import kabuki
from kabuki.hierarchical import Knode
import numpy as np
import pymc as pm
import pandas as pd

class HNodeSimple(kabuki.Hierarchical):
    def create_knodes(self):
        if self.is_group_model:
            mu_g = Knode(pm.Uniform, 'mu_g', lower=-5, upper=5, depends=self.depends['mu'])
            mu_subj = Knode(pm.Normal, 'mu_subj', mu=mu_g, tau=1, depends=('subj_idx',), subj=True)

            like = Knode(pm.Normal, 'like', mu=mu_subj, tau=1, col_name='data', observed=True)

            return [mu_g, mu_subj, like]

        else:
            mu_subj = Knode(pm.Uniform, 'mu_subj', lower=-5, upper=5, depends=self.depends['mu'])

            like = Knode(pm.Normal, 'like', mu=mu_subj, tau=1, col_name='data', observed=True)

            return [mu_subj, like]

class HNodeSimpleVar(kabuki.Hierarchical):
    def create_knodes(self):
        if self.is_group_model:
            mu_g = Knode(pm.Uniform, 'mu_g', lower=-5, upper=5, depends=self.depends['mu'])
            mu_std = Knode(pm.Uniform, 'mu_std', lower=1e-8, upper=100, depends=self.depends['mu_std'])
            mu_tau = Knode(pm.Deterministic, 'mu_tau', doc='mu_tau', eval=lambda x: x**-2, x=mu_std, plot=False, trace=False)
            #v_tau = Knode(pm.Lambda, 'v_tau', lam_fun=lambda x=v_std: x**-2, plot=False, trace=False)
            mu_subj = Knode(pm.Normal, 'mu_subj', mu=mu_g, tau=mu_tau, subj=True)

            like = Knode(pm.Normal, 'like', mu=mu_subj, tau=1, col_name='data', observed=True)

            return [mu_g, mu_std, mu_tau, mu_subj, like]

        else:
            mu_subj = Knode(pm.Uniform, 'mu_subj', lower=-5, upper=5, depends=self.depends['mu'])

            like = Knode(pm.Normal, 'like', mu=mu_subj, tau=1, col_name='data', observed=True)

            return [mu_subj, like]

class HNodeTransform(kabuki.Hierarchical):
    def create_knodes(self):
        if self.is_group_model:
            mu_g = Knode(pm.Uniform, 'mu_g', lower=-5, upper=5, depends=self.depends['mu'], hidden=True)
            mu_std = Knode(pm.Uniform, 'mu_std', lower=1e-8, upper=100, depends=self.depends['mu_std'])
            mu_tau = Knode(pm.Deterministic, 'mu_tau', doc='mu_tau', eval=lambda x: x**-2, x=mu_std, plot=False, trace=False)
            mu_subj = Knode(pm.Normal, 'mu_subj', mu=mu_g, tau=mu_tau, subj=True, plot=False, hidden=True)
            mu_subj_trans = Knode(pm.Deterministic, 'mu_subj_trans', eval=lambda x: x, x=mu_subj, plot=True, trace=True)
            like = Knode(pm.Normal, 'like', mu=mu_subj_trans, tau=1, col_name='data', observed=True)

            return [mu_g, mu_std, mu_tau, mu_subj, mu_subj_trans, like]

        else:
            mu_subj = Knode(pm.Uniform, 'mu_subj', lower=-5, upper=5, depends=self.depends['mu'])
            mu_subj_trans = Knode(pm.Deterministic, 'mu_subj_trans', eval=lambda x: x, x=mu_subj, plot=True, trace=True)

            like = Knode(pm.Normal, 'like', mu=mu_subj_trans, tau=1, col_name='data', observed=True)

            return [mu_subj, mu_subj_trans, like]


def create_test_models():
    n_subj = 5
    data, params = kabuki.generate.gen_rand_data(pm.Normal, {'A':{'mu':0, 'tau':1}, 'B': {'mu':0, 'tau':1}}, subjs=n_subj)
    data = pd.DataFrame(data)
    data['condition2'] = np.random.randint(2, size=len(data))
    models = []

    models.append(HNodeSimple(data['data']))
    models.append(HNodeSimple(data))
    models.append(HNodeSimple(data, depends_on={'mu': 'condition'}))
    models.append(HNodeSimpleVar(data, depends_on={'mu': 'condition'}))
    models.append(HNodeSimpleVar(data, depends_on={'mu': 'condition'}, is_group_model=False))
    models.append(HNodeSimpleVar(data, depends_on={'mu': 'condition', 'mu_std':'condition'}))
    models.append(HNodeSimpleVar(data, depends_on={'mu': 'condition', 'mu_std':'condition2'}))
    models.append(HNodeSimpleVar(data, depends_on={'mu': ['condition', 'condition2'], 'mu_std':'condition2'}))
    models.append(HNodeTransform(data, depends_on={'mu': 'condition', 'mu_std':'condition2'}))

    return models, params

def sample_from_models(models, n_iter = 200):
    """sample from all models"""
    for i, model in enumerate(models):
        print "sample model", i
        model.sample(n_iter)



import kabuki
import pymc as pm
import numpy as np

data = kabuki.utils.generate_effect_data(2, 1, .5, -1.5, 15, 50)

# Generate model
model = kabuki.ANOVA(data, is_subj_model=True, depends_on={'effect':['cond']})
model.mcmc(map_=False)
print model.summary()



import unittest
import kabuki
import numpy as np
import pymc as pm
import math
import scipy as sc
from pprint import pprint
from numpy.random import randn
from numpy import array, sqrt
from nose import SkipTest
from pandas import DataFrame
from time import time


def multi_normal_like(values, vec_mu, tau):
    """logp for multi normal"""
    logp = 0
    for i in xrange(len(vec_mu)):
        logp += pm.normal_like(values[i,:], vec_mu[i], tau)

    return logp

MN = pm.stochastic_from_dist(name="MultiNormal", logp=multi_normal_like, dtype=np.double, mv=True)



class TestStepMethods(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestStepMethods, self).__init__(*args, **kwargs)
        self.uniform_lb = 1e-10
        self.uniform_ub = 1e10

    def runTest(self):
        return


    def assert_results(self, node, true_value, true_mean, true_std=None,
                       mean_tol=0.1, std_tol=0.2):
        """check if the sampler output agree with the analytical meand and
        analytical std
        Input:
            Node - the node to check
            true_value - the true value of the node
            true_mean - the true mean
            true_std - the std of the distribution (None if it's unknown)
            mean_tol - the tolerance to use when checking the difference between
                the true_mean and the sampled mean
            std_tol - same as mean_tol but for checking the std
        """

        pprint(node.stats())
        lb = node.stats()['quantiles'][2.5]
        ub = node.stats()['quantiles'][97.5]
        if not (lb <  true_value < ub):
            print "Warnnig!!!!, sigma was not found in the credible set"


        print "true value:     ", true_value
        print "sampled median: ", node.stats()['quantiles'][50]
        print "sampled mean:   ", node.stats()['mean']
        print "true mean:      ", true_mean
        if true_std is not None:
            print "true std:       ", true_std
            print "sampled std:    ", node.stats()['standard deviation']

        np.testing.assert_allclose(node.stats()['mean'], true_mean, rtol=mean_tol)
        if true_std is not None:
            np.testing.assert_allclose(node.stats()['standard deviation'], true_std, rtol=std_tol)


    def normal_normal(self, add_shift, sigma_0, sigma_beta, sigma_y, true_mu,
                     n_subjs, avg_samples, seed, use_metropolis):
        """check the normal_normal configuration
        Model:
        x ~ N(mu_0, sigma_0**-2)
        y ~ N(x + b, sigma_y**-2)
        only mu is Stochastic

        b is constant in the model. it is generated from N(o,sigma_b**-2)
        y is generated from N(true_mu + b, sigma_y**-2)
            add_shift - whether to add some

        n_subjs - number of b
        avg_samples - the average samples per subject

        """

        np.random.seed(seed)

        #create nodes
        tau_0 = sigma_0**-2
        mu_0 = 0.
        nodes, size, x_values = \
        self.create_nodes_for_normal_normal(add_shift, tau_0, mu_0, sigma_beta,
                                       sigma_y, true_mu, n_subjs, avg_samples)
        mu = nodes['mu']

        #sample
        mm = pm.MCMC(nodes)
        if use_metropolis:
            mm.sample(20000,5000)
        else:
            mm.use_step_method(kabuki.steps.kNormalNormal,mu)#, b=b)
            mm.sample(10000)

        #calc the new distrbution
        total_n = sum(size)
        tau = sigma_y**-2
        sum_obs = sum([sum(x.value.flatten()) for x in mm.observed_stochastics])
        if add_shift:
            tmp = sum(array(size)* x_values)
        else:
            tmp = 0

        tau_prime = tau_0 + total_n*tau
        mu_prime = (tau*(sum_obs - tmp) + mu_0*tau_0)/tau_prime
        true_std = 1./np.sqrt(tau_prime)

        self.assert_results(mu, true_mu, mu_prime, true_std, mean_tol=0.1, std_tol=0.1)

        return mm, mu_prime, true_std

    def create_nodes_for_normal_normal(self, add_shift, tau_0, mu_0, sigma_beta,
                                       sigma_y, true_mu, n_subjs, avg_samples):
        """ create the normal normal nodes"""

        mu = pm.Normal('mu',mu_0,tau_0)
        nodes = {'mu': mu}
        size = [None]*n_subjs
        x_values = [None]*n_subjs
        if add_shift:
            b = []
        else:
            b = None
        for i in range(n_subjs):
            size[i] = int(max(1, avg_samples + randn()*10))
            if add_shift:
                x_values[i] = randn()*sigma_beta
                value = randn(size[i]) * sigma_y + true_mu + x_values[i]
                x = pm.Lambda('x%d' % i, lambda x=x_values[i]:x)
                y = pm.Normal('y%d' % i,mu+x, sigma_y**-2, value=value,observed=True)
                nodes['x%d' % i] = x
                b.append(x)
            else:
                value = randn(size[i]) * sigma_y + true_mu
                y = pm.Normal('y%d' % i,mu, sigma_y**-2, value=value,observed=True)

            nodes['y%d' % i] = y

        return nodes, size, x_values


    def normal_normal_bundle(self, use_metropolis):
        """run normal_normal with different parameters"""
        self.normal_normal(add_shift=True, sigma_0=100., sigma_beta=2., sigma_y=1.5,
                         true_mu=-3., n_subjs=1, avg_samples=100, seed=1, use_metropolis=use_metropolis)
        self.normal_normal(add_shift=True, sigma_0=50., sigma_beta=3., sigma_y=2,
                         true_mu=-2., n_subjs=2, avg_samples=10, seed=2, use_metropolis=use_metropolis)
        self.normal_normal(add_shift=True, sigma_0=10., sigma_beta=1., sigma_y=2.5,
                         true_mu=-1., n_subjs=3, avg_samples=10, seed=3, use_metropolis=use_metropolis)
        self.normal_normal(add_shift=False, sigma_0=1., sigma_beta=0.5, sigma_y=0.5,
                         true_mu=-4., n_subjs=4, avg_samples=50, seed=4, use_metropolis=use_metropolis)
        self.normal_normal(add_shift=False, sigma_0=50., sigma_beta=0.3, sigma_y=1.5,
                         true_mu=-6., n_subjs=5, avg_samples=50, seed=5, use_metropolis=use_metropolis)
        self.normal_normal(add_shift=False, sigma_0=100., sigma_beta=0.75, sigma_y=2.5,
                         true_mu=100., n_subjs=6, avg_samples=30, seed=6, use_metropolis=use_metropolis)

    def test_normal_normal_solution(self):
        """test normal normal analytic solution"""
        self.normal_normal_bundle(use_metropolis=True)


    def test_kNormalNormal(self):
        """test normal_normal step method"""
        self.normal_normal_bundle(use_metropolis=False)


    def create_nodes_for_PriorNormalstd(self, n_subjs, sigma_0, mu_0, prior, **kwargs):
        """"create node for models with PriorNormalstd step method"""
        #create nodes
        if prior is pm.Uniform:
            sigma = pm.Uniform('sigma', self.uniform_lb, self.uniform_ub, value=1.)
        elif prior is kabuki.utils.HalfCauchy:
            sigma = kabuki.utils.HalfCauchy('sigma', **kwargs)

        x_values = [None]*n_subjs
        nodes = {'sigma': sigma}
        for i in range(n_subjs):
            x_values[i] = randn()*sigma_0 + mu_0
            x = pm.Normal('x%d' % i, mu_0, sigma**-2, value=x_values[i], observed=True)
            nodes['x%d' % i] = x
        return nodes, x_values


    def uniform_normalstd(self, sigma_0, mu_0, n_subjs, seed, use_metropolis):
        """test estimation of Normal distribution std with uniform prior
            sigma_0 - the value of the std noe
            mu_0 - the value of the mu node
            use_metropolis - should it use metropolis to evaluate the sampled mean
                instead of the UniformPriorNormalstd
        """

        np.random.seed(seed)

        nodes, x_values = self.create_nodes_for_PriorNormalstd(n_subjs, sigma_0, mu_0, prior=pm.Uniform)
        sigma = nodes['sigma']
        mm = pm.MCMC(nodes)

        if use_metropolis:
            mm.sample(20000,5000)
        else:
            mm.use_step_method(kabuki.steps.UniformPriorNormalstd, sigma)
            mm.sample(10000)



        #calc the new distrbution
        alpha = (n_subjs - 1) / 2.
        beta  = sum([(x - mu_0)**2 for x in x_values]) / 2.
        true_mean = math.gamma(alpha-0.5)/math.gamma(alpha)*np.sqrt(beta)
        anal_var = beta / (alpha - 1) - true_mean**2
        true_std = np.sqrt(anal_var)

        self.assert_results(sigma, sigma_0, true_mean, true_std)
        return mm


    def test_uniform_normalstd_numerical_solution(self):
        """test uniform_normalstd with Metropolis to evaluate the numerical solution of
        the mean and std"""
        self.uniform_normalstd(sigma_0=0.5, mu_0=0, n_subjs=8, seed=1, use_metropolis=True)
        self.uniform_normalstd(sigma_0=1.5, mu_0=-100, n_subjs=4, seed=2, use_metropolis=True)
        self.uniform_normalstd(sigma_0=2.5, mu_0=2, n_subjs=5, seed=3, use_metropolis=True)
        self.uniform_normalstd(sigma_0=3.5, mu_0=-4, n_subjs=7, seed=4, use_metropolis=True)
#        self.uniform_normalstd(sigma_0=4.5, mu_0=10, n_subjs=4, seed=5, use_metropolis=True)

    def test_UniformNormalstd_step_method(self):
        """test UniformPriorNormalstd step method"""
        self.uniform_normalstd(sigma_0=0.5, mu_0=0, n_subjs=8, seed=1, use_metropolis=False)
        self.uniform_normalstd(sigma_0=1.5, mu_0=-100, n_subjs=4, seed=2, use_metropolis=False)
        self.uniform_normalstd(sigma_0=2.5, mu_0=2, n_subjs=5, seed=3, use_metropolis=False)
        self.uniform_normalstd(sigma_0=3.5, mu_0=-4, n_subjs=7, seed=4, use_metropolis=False)
        self.uniform_normalstd(sigma_0=4.5, mu_0=10, n_subjs=4, seed=5, use_metropolis=False)


    def numerical_solution(self, defective_posterior, lb, ub):
        """numerical estimation of the mean and std from defective posterior
        defective_posterior <func> - the defective posterior
        lb - lower bound
        ub - upper bound
        """

        norm_factor = sc.integrate.quad(defective_posterior,lb,ub)[0]
        #function to compute moments
        moment = lambda x,n=1: defective_posterior(x) * (x**n) / norm_factor

        #computing mean and std
        true_mean = sc.integrate.quad(moment,lb, ub, args=(1))[0]
        m2 = sc.integrate.quad(moment,lb, ub, args=(2))[0]
        anal_var = m2 - true_mean**2
        true_std = np.sqrt(anal_var)
        return true_mean, true_std


    def half_cauchy_normal_std(self, sigma_0=1., mu_0=0., S=10, n_subjs=8, seed=1,
                            use_metropolis=False):
        """test estimation of Normal distribution std with halh Cauchy prior
            sigma_0 - the value of the std noe
            mu_0 - the value of the mu node
            S - the scale of the half Cauchy
            use_metropolis - should it use metropolis to evaluate the sampled mean
                instead of the UniformPriorNormalstd
        """


        #create model
        np.random.seed(seed)
        nodes, x_values = \
        self.create_nodes_for_PriorNormalstd(n_subjs, sigma_0, mu_0,
                                             prior=kabuki.utils.HalfCauchy,
                                             S=S, value=1)
        sigma = nodes['sigma']

        #sample
        mm = pm.MCMC(nodes)
        if use_metropolis:
            mm.sample(20000,5000)
        else:
            mm.use_step_method(kabuki.steps.HCauchyPriorNormalstd, sigma)
            mm.sample(10)

        #compute defective posterior
        beta = sum((array(x_values) - mu_0)**2)/2
        def defective_posterior(x, n=n_subjs, beta=beta, S=S):
           gammapdf = (x**2)**(-n/2.) * np.exp(-beta/(x**2))
           cauchy = S / (x**2 + S**2)
           return gammapdf * cauchy

        #check results
        true_mean, true_std = self.numerical_solution(defective_posterior, 0, np.inf)
        self.assert_results(sigma, sigma_0, true_mean, true_std)
        return mm

    def half_cauchy_bundle(self, use_metropolis):
        self.half_cauchy_normal_std(sigma_0=4.5, mu_0=0, n_subjs=8, seed=1, S=5,
                                    use_metropolis=use_metropolis)
        self.half_cauchy_normal_std(sigma_0=0.5, mu_0=-100, n_subjs=4, seed=2, S=20,
                                    use_metropolis=use_metropolis)
        self.half_cauchy_normal_std(sigma_0=5.5, mu_0=2, n_subjs=5, seed=3, S=3,
                                    use_metropolis=use_metropolis)
        self.half_cauchy_normal_std(sigma_0=1.5, mu_0=-4, n_subjs=7, seed=4, S=10,
                                    use_metropolis=use_metropolis)
        self.half_cauchy_normal_std(sigma_0=4.5, mu_0=10, n_subjs=4, seed=5, S=15,
                                    use_metropolis=use_metropolis)

    def test_half_cauchy_numerical_solution(self):
        """test half_cauchy_normal_std with Metropolis to evaluate the numerical solution of
        the mean and std"""
        self.half_cauchy_bundle(use_metropolis=True)

    def test_HCauchyNormalstd_step_method(self):
        """test HCauchy step method"""
        raise SkipTest("The HCauchy gibbs step method does not work.")
        self.half_cauchy_bundle(use_metropolis=False)

    def run_SPXcentered(self, sigma_x, n_subjs, size, mu_value, mu_step_method, seed):
        """ run a single Spxcentered test"""

        #init basic  mcmc
        if np.isscalar(mu_value):
            n_conds = 1
        else:
            n_conds = len(mu_value)

        max_tries = 5
        iter = 10000 #100000
        burnin= 5000 #90000
        nodes, t_values = self.create_hierarchical_model(sigma_x=sigma_x, n_subjs=n_subjs, size=size,
                                                         mu_value=mu_value, seed=seed)
        mcmc = pm.MCMC(nodes)
        [mcmc.use_step_method(mu_step_method, node) for node in nodes['mu']]

        #init mcmc with SPX step method
        nodes_spx, t_values = self.create_hierarchical_model(sigma_x=sigma_x, n_subjs=n_subjs, size=size,
                                                             mu_value=mu_value, seed=seed)
        mcmc_spx = pm.MCMC(nodes_spx)
        mcmc_spx.use_step_method(kabuki.steps.SPXcentered, loc=nodes_spx['mu'],
                                 scale=nodes_spx['sigma'],
                                 loc_step_method=mu_step_method)


        #init mcmc with spx on vec model
        nodes_vpx, t_values = self.create_hierarchical_model(sigma_x=sigma_x, n_subjs=n_subjs, size=size,
                                                             mu_value=mu_value, seed=seed, vec=True)
        mcmc_vpx = pm.MCMC(nodes_vpx)
        mcmc_vpx.use_step_method(kabuki.steps.SPXcentered, loc=nodes_vpx['mu'],
                                 scale=nodes_vpx['sigma'],
                                 loc_step_method=mu_step_method)


        #run all the models until they converge to the same values
        i_try = 0
        while i_try < max_tries:
            print "~~~~~ trying for the %d time ~~~~~~" % (i_try + 1)

            #run spx mcmc
            i_t = time()
            mcmc_spx.sample(iter,burnin)
            print "spx sampling took %.2f seconds" % (time() - i_t)
            stats = dict([('mu%d spx' %x, mcmc_spx.mu[x].stats()) for x in range(n_conds)])

            #run vpx mcmc
            i_t = time()
            mcmc_vpx.sample(iter,burnin)
            print "vpx sampling took %.2f seconds" % (time() - i_t)
            stats.update(dict([('mu%d vpx' %x, mcmc_vpx.mu[x].stats()) for x in range(n_conds)]))

            #run basic mcmc
            i_t = time()
            mcmc.sample(iter,burnin)
            print "basic sampling took %.2f seconds" % (time() - i_t)
            stats.update(dict([('mu%d basic' %x, mcmc.mu[x].stats()) for x in range(n_conds)]))

            df = DataFrame(stats, index=['mean', 'standard deviation']).T
            df = df.rename(columns = {'mean':'mean', 'standard deviation': 'std'})
            print df

            #check if all the results are close enough
            try:
                for i in range(len(df)/3):
                    np.testing.assert_allclose(df[(3*i+0):(3*i+1)], df[(3*i+1):(3*i+2)], atol=0.1, rtol=0.01)
                    np.testing.assert_allclose(df[(3*i+1):(3*i+2)], df[(3*i+2):(3*i+3)], atol=0.1, rtol=0.01)
                    np.testing.assert_allclose(df[(3*i+2):(3*i+3)], df[(3*i+0):(3*i+1)], atol=0.1, rtol=0.01)

                break
            #if not add more runs
            except AssertionError:
                print "Failed to reach agreement. trying again"
                i_try += 1

        assert (i_try < max_tries), "could not replicate values using different mcmc samplers"


    def test_SPX(self):
        """test a bundle of SPXcentered tests"""
        print "*************** Test 1 ***************"
        self.run_SPXcentered(sigma_x=1, n_subjs=5, size=100, mu_value=4,
                             mu_step_method=kabuki.steps.kNormalNormal, seed=1)
        print "*************** Test 2 ***************"
        self.run_SPXcentered(sigma_x=1, n_subjs=5, size=10, mu_value=(4,3,2,1,0,4,3,2,1,0),
                             mu_step_method=kabuki.steps.kNormalNormal, seed=1)
        print "*************** Test 3 ***************"
        self.run_SPXcentered(sigma_x=0.5, n_subjs=5, size=10, mu_value=(4,3),
                             mu_step_method=kabuki.steps.kNormalNormal, seed=1)
        print "*************** Test 4 ***************"
        self.run_SPXcentered(sigma_x=0.1, n_subjs=5, size=10, mu_value=(4,3),
                             mu_step_method=kabuki.steps.kNormalNormal, seed=1)
        print "*************** Test 5 ***************"
        self.run_SPXcentered(sigma_x=1, n_subjs=5, size=10, mu_value=range(20),
                             mu_step_method=kabuki.steps.kNormalNormal, seed=1)
        print "*************** Test 6 ***************"
        self.run_SPXcentered(sigma_x=0.1, n_subjs=5, size=10, mu_value=range(20),
                             mu_step_method=kabuki.steps.kNormalNormal, seed=1)

    def create_hierarchical_model(self, sigma_x=1, n_subjs=5, size=100, mu_value=4, seed=1, vec=False):
        """
        create an hierarchical normal model
        y_ijk ~ N(x_jk, 1) (i sample, j subject, k condition)
        x_jk ~ N(m_k, sigma_x**2)
        m_k ~ N(0, 100*-2)

        Input:
            m_value <list> - m_value[k] - m_k
            size <int>- number of samples per subject per category
            n_subjs <int> - no. of subjects
            vec <boolean> - use a vectorized model
        """
        #init
        np.random.seed(seed)
        if np.isscalar(mu_value):
            mu_value = [mu_value]
        n_conds = len(mu_value)
        mu = [None] * n_conds
        subj_nodes = [None]*n_conds
        data_nodes = [None]*n_conds

        #true_values
        true_values = {}
        true_values['sigma'] = sigma_x
        true_values['mu'] = mu_value

        #init sigma node
        sigma = pm.Uniform('sigma', 1e-10,1e10, value=1)
        tau = sigma**-2

        #create nodes for each cond
        for i_cond in range(n_conds):
            #initalize the true value of x
            true_x = randn(n_subjs)*sigma_x + mu_value[i_cond]
            value = np.random.randn(n_subjs, size).T + true_x
            value = value.T
            print true_x

            #init mu and sigma
            mu[i_cond] = pm.Normal('mu%d' % i_cond, 0, 100.**-2, value=0)

            #create subj_nodes (x + y)
            if vec:
                subj_nodes[i_cond] = pm.Normal('x%d' % (i_cond), mu[i_cond], tau, size=n_subjs)
                data_nodes[i_cond] = MN('y%d' % (i_cond), vec_mu=subj_nodes[i_cond], tau=1, value=value, observed=True)
            else:
                subj_nodes[i_cond] = [None]*n_subjs
                data_nodes[i_cond] = [None]*n_subjs
                for i_subj in range(n_subjs):
                    #x is generate from the mean.
                    subj_nodes[i_cond][i_subj] = pm.Normal('x%d_%d' % (i_cond, i_subj), mu[i_cond], tau)
                    data_nodes[i_cond][i_subj] = pm.Normal('y%d_%d' % (i_cond, i_subj),
                                                           mu=subj_nodes[i_cond][i_subj],
                                                           tau=1, value=value[i_subj,:], observed=True)

        #create nodes dictionary
        nodes = {}
        nodes['x'] = subj_nodes
        nodes['y'] = data_nodes
        nodes['mu'] = mu
        nodes['sigma'] = sigma

        return nodes, true_values


    def run_SliceStep(self, sigma_x, n_subjs, size, mu_value, seed, left, max_tries=5):

        #init basic  mcmc
        if np.isscalar(mu_value):
            n_conds = 1
        else:
            n_conds = len(mu_value)

        iter = 10000 #100000
        burnin= 5000 #90000

        #init basic mcmc
        nodes, t_values = self.create_hierarchical_model(sigma_x=sigma_x, n_subjs=n_subjs, size=size,
                                                         mu_value=mu_value, seed=seed)
        mcmc = pm.MCMC(nodes)
        [mcmc.use_step_method(kabuki.steps.kNormalNormal, node) for node in nodes['mu']]

        #init mcmc with slice step
        nodes_s, t_values = self.create_hierarchical_model(sigma_x=sigma_x, n_subjs=n_subjs, size=size,
                                                         mu_value=mu_value, seed=seed)
        mcmc_s = pm.MCMC(nodes_s)
        [mcmc_s.use_step_method(kabuki.steps.kNormalNormal, node) for node in nodes_s['mu']]
        mcmc_s.use_step_method(kabuki.steps.SliceStep, nodes_s['sigma'], width=3, left=left)


        #run all the models until they converge to the same values
        i_try = 0
        stats = {}
        while i_try < max_tries:
            print "~~~~~ trying for the %d time ~~~~~~" % (i_try + 1)

            #run slice mcmc
            i_t = time()
            mcmc_s.sample(iter,burnin)
            print "slice sampling took %.2f seconds" % (time() - i_t)
            stats.update(dict([('mu%d S' %x, mcmc_s.mu[x].stats()) for x in range(n_conds)]))

            #run basic mcmc
            i_t = time()
            mcmc.sample(iter,burnin)
            print "basic sampling took %.2f seconds" % (time() - i_t)
            stats.update(dict([('mu%d basic' %x, mcmc.mu[x].stats()) for x in range(n_conds)]))

            df = DataFrame(stats, index=['mean', 'standard deviation']).T
            df = df.rename(columns = {'mean':'mean', 'standard deviation': 'std'})
            print df

            #check if all the results are close enough
            try:
                for i in range(len(df)/2):
                    np.testing.assert_allclose(df[(2*i+0):(2*i+1)], df[(2*i+1):(2*i+2)], atol=0.1, rtol=0.01)
                break
            #if not add more runs
            except AssertionError:
                print "Failed to reach agreement In:"
                print df[(2*i):(2*(i+1))]
                print "trying again"

            i_try += 1

        assert (i_try < max_tries), "could not replicate values using different mcmc samplers"

        return mcmc, mcmc_s

    def test_SliceStep(self):
        """test a bundle of SPXcentered tests"""
        print "*************** Test 1 ***************"
        self.run_SliceStep(sigma_x=1, n_subjs=5, size=100, mu_value=4, seed=1)
        print "*************** Test 2 ***************"
        self.run_SliceStep(sigma_x=1, n_subjs=5, size=10, mu_value=range(10), seed=1)
        print "*************** Test 3 ***************"
        self.run_SliceStep(sigma_x=0.5, n_subjs=5, size=10, mu_value=(4,3), seed=1)
        print "*************** Test 4 ***************"
        self.run_SliceStep(sigma_x=0.1, n_subjs=5, size=10, mu_value=(4,3), seed=1)
        print "*************** Test 5 ***************"
        self.run_SliceStep(sigma_x=1, n_subjs=5, size=10, mu_value=range(20), seed=1)
        print "*************** Test 6 ***************"
        self.run_SliceStep(sigma_x=0.1, n_subjs=5, size=10, mu_value=range(20), seed=1)
        print "*************** Test 7 ***************"
        self.run_SliceStep(sigma_x=0.1, n_subjs=5, size=10, mu_value=(4,3), seed=1, left=0)
        print "*************** Test 8 ***************"
        self.run_SliceStep(sigma_x=1, n_subjs=5, size=10, mu_value=range(20), seed=1, left=0)
        print "*************** Test 9 ***************"
        self.run_SliceStep(sigma_x=0.1, n_subjs=5, size=10, mu_value=range(20), seed=1, left=0)



import warnings
import numpy as np
import pymc as pm

from pymc import Stochastic, utils
from pymc.distributions import bind_size

# To remain compatibility with pymc 2.1, we copy this function.
# Once pymc 2.2 is released this should be depracated.

def debug_wrapper(func, name):
    # Wrapper to debug distributions

    import pdb

    def wrapper(*args, **kwargs):

        print('Debugging inside %s:' % name)
        print('\tPress \'s\' to step into function for debugging')
        print('\tCall \'args\' to list function arguments')

        # Set debugging trace
        pdb.set_trace()

        # Call function
        return func(*args, **kwargs)

    return wrapper


def new_dist_class(*new_class_args):
    """
    Returns a new class from a distribution.

    :Parameters:
      dtype : numpy dtype
        The dtype values of instances of this class.
      name : string
        Name of the new class.
      parent_names : list of strings
        The labels of the parents of this class.
      parents_default : list
        The default values of parents.
      docstr : string
        The docstring of this class.
      logp : function
        The log-probability function for this class.
      random : function
        The random function for this class.
      mv : boolean
        A flag indicating whether this class represents array-valued
        variables.

      .. note::
        stochastic_from_dist provides a higher-level version.

      :SeeAlso:
        stochastic_from_dist
    """

    (dtype, name, parent_names, parents_default, docstr, logp, random, mv, logp_partial_gradients) = new_class_args

    class new_class(Stochastic):
        __doc__ = docstr

        def __init__(self, *args, **kwds):
            (dtype, name, parent_names, parents_default, docstr, logp, random, mv, logp_partial_gradients) = new_class_args
            parents=parents_default

            # Figure out what argument names are needed.
            arg_keys = ['name', 'parents', 'value', 'observed', 'size', 'trace', 'rseed', 'doc', 'debug', 'plot', 'verbose']
            arg_vals = [None, parents, None, False, None, True, True, None, False, None, -1]
            if 'isdata' in kwds:
                warnings.warn('"isdata" is deprecated, please use "observed" instead.')
                kwds['observed'] = kwds['isdata']
                pass


            # No size argument allowed for multivariate distributions.
            if mv:
                arg_keys.pop(4)
                arg_vals.pop(4)

            arg_dict_out = dict(zip(arg_keys, arg_vals))
            args_needed = ['name'] + parent_names + arg_keys[2:]

            # Sort positional arguments
            for i in xrange(len(args)):
                try:
                    k = args_needed.pop(0)
                    if k in parent_names:
                        parents[k] = args[i]
                    else:
                        arg_dict_out[k] = args[i]
                except:
                    raise ValueError('Too many positional arguments provided. Arguments for class ' + self.__class__.__name__ + ' are: ' + str(args_needed))


            # Sort keyword arguments
            for k in args_needed:
                if k in parent_names:
                    try:
                        parents[k] = kwds.pop(k)
                    except:
                        if k in parents_default:
                            parents[k] = parents_default[k]
                        else:
                            raise ValueError('No value given for parent ' + k)
                elif k in arg_dict_out.keys():
                    try:
                        arg_dict_out[k] = kwds.pop(k)
                    except:
                        pass

            # Remaining unrecognized arguments raise an error.
            if len(kwds) > 0:
                raise TypeError('Keywords '+ str(kwds.keys()) + ' not recognized. Arguments recognized are ' + str(args_needed))

        # Determine size desired for scalar variables.
        # Notes
        # -----
        # Case | init_val     | parents       | size | value.shape | bind size
        # ------------------------------------------------------------------
        # 1.1  | None         | scalars       | None | 1           | 1
        # 1.2  | None         | scalars       | n    | n           | n
        # 1.3  | None         | n             | None | n           | 1
        # 1.4  | None         | n             | n(m) | n (Error)   | 1 (-)
        # 2.1  | scalar       | scalars       | None | 1           | 1
        # 2.2  | scalar       | scalars       | n    | n           | n
        # 2.3  | scalar       | n             | None | n           | 1
        # 2.4  | scalar       | n             | n(m) | n (Error)   | 1 (-)
        # 3.1  | n            | scalars       | None | n           | n
        # 3.2  | n            | scalars       | n(m) | n (Error)   | n (-)
        # 3.3  | n            | n             | None | n           | 1
        # 3.4  | n            | n             | n(m) | n (Error)   | 1 (-)

            if not mv:

                shape = arg_dict_out.pop('size')
                shape = None if shape is None else tuple(np.atleast_1d(shape))

                init_val = arg_dict_out['value']
                init_val_shape = None if init_val is None else np.shape(init_val)

                if len(parents) > 0:
                    pv = [np.shape(utils.value(v)) for v in parents.values()]
                    biggest_parent = np.argmax([(np.prod(v) if v else 0) for v in pv])
                    parents_shape = pv[biggest_parent]

                    # Scalar parents can support any shape.
                    if np.prod(parents_shape) < 1:
                        parents_shape = None

                else:
                    parents_shape = None

                def shape_error():
                    raise ValueError('Shapes are incompatible: value %s, largest parent %s, shape argument %s'%(shape, init_val_shape, parents_shape))

                if init_val_shape is not None and shape is not None and init_val_shape != shape:
                    shape_error()

                given_shape = init_val_shape or shape
                bindshape = given_shape or parents_shape

                # Check consistency of bindshape and parents_shape
                if parents_shape is not None:
                    # Uncomment to leave broadcasting completely up to NumPy's random functions
                    # if bindshape[-np.alen(parents_shape):]!=parents_shape:
                    # Uncomment to limit broadcasting flexibility to what the Fortran likelihoods can handle.
                    if bindshape<parents_shape:
                        shape_error()

                if random is not None:
                    random = bind_size(random, bindshape)


            elif 'size' in kwds.keys():
                raise ValueError('No size argument allowed for multivariate stochastic variables.')


            # Call base class initialization method
            if arg_dict_out.pop('debug'):
                logp = debug_wrapper(logp)
                random = debug_wrapper(random)
            else:
                Stochastic.__init__(self, logp=logp, random=random, dtype=dtype, **arg_dict_out)

    new_class.__name__ = name
    new_class.parent_names = parent_names
    new_class.parents_default = parents_default
    new_class.dtype = dtype
    new_class.mv = mv
    new_class.raw_fns = {'logp': logp, 'random': random}

    return new_class

def scipy_stochastic(scipy_dist, **kwargs):
    """
    Return a Stochastic subclass made from a particular SciPy distribution.
    """
    import inspect
    import scipy.stats.distributions as sc_dst
    from pymc.ScipyDistributions import separate_shape_args

    if scipy_dist.__class__.__name__.find('_gen'):
        scipy_dist = scipy_dist(**kwargs)

    name = scipy_dist.__class__.__name__.replace('_gen','').capitalize()

    (args, varargs, varkw, defaults) = inspect.getargspec(scipy_dist._pdf)

    shape_args = args[2:]
    if isinstance(scipy_dist, sc_dst.rv_continuous):
        dtype=float

        def logp(value, **kwds):
            args, zkwds = separate_shape_args(kwds, shape_args)
            if hasattr(scipy_dist, '_logp'):
                return scipy_dist._logp(value, *args)
            else:
                return np.sum(scipy_dist.logpdf(value,*args,**kwds))

        parent_names = shape_args + ['loc', 'scale']
        defaults = [None] * (len(parent_names)-2) + [0., 1.]

    elif isinstance(scipy_dist, sc_dst.rv_discrete):
        dtype=int

        def logp(value, **kwds):
            args, kwds = separate_shape_args(kwds, shape_args)
            if hasattr(scipy_dist, '_logp'):
                return scipy_dist._logp(value, *args)
            else:
                return np.sum(scipy_dist.logpmf(value,*args,**kwds))

        parent_names = shape_args + ['loc']
        defaults = [None] * (len(parent_names)-1) + [0]
    else:
        return None

    parents_default = dict(zip(parent_names, defaults))

    def random(shape=None, **kwds):
        args, kwds = separate_shape_args(kwds, shape_args)

        if shape is None:
            return scipy_dist.rvs(*args, **kwds)
        else:
            return np.reshape(scipy_dist.rvs(*args, **kwds), shape)

    # Build docstring from distribution
    docstr = name[0]+' = '+name + '(name, '+', '.join(parent_names)+', value=None, shape=None, trace=True, rseed=True, doc=None)\n\n'
    docstr += 'Stochastic variable with '+name+' distribution.\nParents are: '+', '.join(parent_names) + '.\n\n'
    docstr += """
Methods:

    random()
        - draws random value
          sets value to return value

    ppf(q)
        - percent point function (inverse of cdf --- percentiles)
          sets value to return value

    isf(q)
        - inverse survival function (inverse of sf)
          sets value to return value

    stats(moments='mv')
        - mean('m',axis=0), variance('v'), skew('s'), and/or kurtosis('k')


Attributes:

    logp
        - sum(log(pdf())) or sum(log(pmf()))

    cdf
        - cumulative distribution function

    sf
        - survival function (1-cdf --- sometimes more accurate)

    entropy
        - (differential) entropy of the RV.


NOTE: If you encounter difficulties with this object, please try the analogous
computation using the rv objects in scipy.stats.distributions directly before
reporting the bug.
    """

    new_class = new_dist_class(dtype, name, parent_names, parents_default, docstr, logp, random, True, None)
    class newer_class(new_class):
        __doc__ = docstr
        rv = scipy_dist
        rv.random = random

        def __init__(self, *args, **kwds):
            new_class.__init__(self, *args, **kwds)
            self.args, self.kwds = separate_shape_args(self.parents, shape_args)
            self.frozen_rv = self.rv(self.args, self.kwds)
            self._random = bind_size(self._random, self.shape)

        def _pymc_dists_to_value(self, args):
            """Replace arguments that are a pymc.Node with their value."""
            # This is needed because the scipy rv function transforms
            # every input argument which causes new pymc lambda
            # functions to be generated. Thus, when calling this many
            # many times, excessive amounts of RAM are used.
            new_args = []
            for arg in args:
                if isinstance(arg, pm.Node):
                    new_args.append(arg.value)
                else:
                    new_args.append(arg)

            return new_args

        def pdf(self, value=None):
            """
            The probability distribution function of self conditional on parents
            evaluated at self's current value
            """
            if value is None:
                value = self.value
            return self.rv.pdf(value, *self._pymc_dists_to_value(self.args), **self.kwds)

        def cdf(self, value=None):
            """
            The cumulative distribution function of self conditional on parents
            evaluated at self's current value
            """
            if value is None:
                value = self.value
            return self.rv.cdf(value, *self._pymc_dists_to_value(self.args), **self.kwds)

        def sf(self, value=None):
            """
            The survival function of self conditional on parents
            evaluated at self's current value
            """
            if value is None:
                value = self.value
            return self.rv.sf(self.value, *self._pymc_dists_to_value(self.args), **self.kwds)

        def ppf(self, q):
            """
            The percentile point function (inverse cdf) of self conditional on parents.
            Self's value will be set to the return value.
            """
            self.value = self.rv.ppf(q, *self._pymc_dists_to_value(self.args), **self.kwds)
            return self.value

        def isf(self, q):
            """
            The inverse survival function of self conditional on parents.
            Self's value will be set to the return value.
            """
            self.value = self.rv.isf(q, *self._pymc_dists_to_value(self.args), **self.kwds)
            return self.value

        def stats(self, moments='mv'):
            """The first few moments of self's distribution conditional on parents"""
            return self.rv.stats(moments=moments, *self._pymc_dists_to_value(self.args), **self.kwds)

        def _entropy(self):
            """The entropy of self's distribution conditional on its parents"""
            return self.rv.entropy(*self._pymc_dists_to_value(self.args), **self.kwds)
        entropy = property(_entropy, doc=_entropy.__doc__)

    newer_class.__name__ = new_class.__name__
    return newer_class




import kabuki
from kabuki.hierarchical import Knode
import numpy as np
import pymc as pm

def load_models():
    """
    This function returns a list of models that are going to be tested
    """
    import hddm
    n = 400
    dtype = [('response', np.int), ('rt', np.float), ('subj_idx', np.int32), ('cond1', 'S20'), ('cond2', 'S20')]
    data = np.empty(n, dtype=dtype)
    data['rt'] = np.random.rand(n) + 0.5;
    data['response'] = np.random.randint(2, size=n)
    data['cond1'] = np.array(['A','B'])[np.random.randint(2, size=n)]
    data['cond2'] = np.array(['A','B'])[np.random.randint(2, size=n)]
    data['subj_idx'] = np.zeros(n)

    models = []
    #model 1
    m = hddm.HDDM(data, depends_on = {'v':'cond1'})
    models.append(m)

    #model 2
    m = hddm.HDDM(data, depends_on = {'v':['cond1', 'cond2'], 'a':'cond1'}, include =['z','sv'])
    models.append(m)

    data['subj_idx'] = np.random.randint(5, size=n)
    #model 3
    m = hddm.HDDM(data, depends_on = {'v':'cond1'})
    models.append(m)

    m = hddm.HDDM(data, depends_on = {'v':['cond1', 'cond2'], 'a':'cond1'}, include=['sv'], group_only_nodes=['sv'])
    models.append(m)

    return models

def sample_from_models(models, n_iter = 200):
    """sample from all models"""
    for i, model in enumerate(models):
        print "sample model", i
        model.sample(n_iter)



from __future__ import division

import numpy as np
from copy import copy

def _add_noise(params, check_valid_func=None, bounds=None, noise=.1, exclude_params=()):
    """Add individual noise to each parameter.

        :Arguments:
            params : dict
                Parameter names and values

        :Optional:
            check_valid_func : function <default lambda x: True>
                Function that takes as input the parameters as kwds
                and returns True if param values are admissable.
            bounds : dict <default={}>
                Dict containing parameter names and
                (lower, upper) value for valid parameter
                range.
            noise : float <default=.1>
                Standard deviation of random gaussian
                variable to add to each parameter.
            exclude_params : tuple <default=()>
                Do not add noise to these parameters.

        :Returns:
            params : dict
                parameters with noise added.

    """

    def sample_value(param, value):
        if np.isscalar(noise):
            param_noise = noise
        else:
            param_noise = noise[param]

        if param in bounds:
            while True:
                sampled_value = np.random.normal(loc=value, scale=param_noise)
                if sampled_value >= bounds[param][0] and sampled_value <= bounds[param][1]:
                    return sampled_value
        else:
            return np.random.normal(loc=value, scale=noise)

    if bounds is None:
        bounds = {}

    if check_valid_func is None:
        check_valid_func = lambda **params: True

    # Sample parameters until accepted
    while True:
        params_cur = copy(params)

        for param, value in params_cur.iteritems():
            if param not in exclude_params:
                params_cur[param] = sample_value(param, value)

        if check_valid_func(**params_cur):
            return params_cur

def gen_rand_data(Stochastic, params, size=50, subjs=1, subj_noise=.1, exclude_params=(),
                  column_name='data', check_valid_func=None, bounds=None, seed=None):
    """Generate a random dataset using a user-defined random distribution.

    :Arguments:
        Stochastic : a pymc stochastic class of the target distribution (e.g., pymc.Normal)
        params : dict
            Parameters to use for data generation. Two options possible:
                * {'param1': value, 'param2': value2}
                * {'condition1': {'param1': value, 'param2': value2},
                   'condition2': {'param1': value3, 'param2': value4}}
            In the second case, the dataset is generated with multiple conditions
            named after the key and will be sampled using the corresponding parameters.

    :Optional:
        size : int <default: 50>
            How many values to sample for each condition for each subject.
        subjs : int <default: 1>
            How many subjects to generate data from. Individual subject parameters
            will be normal distributed around the provided parameters with variance
            subj_noise if subjs > 1. If only one subject is simulated no noise is added.
        subj_noise : float or dictionary <default: .1>
            How much to perturb individual subj parameters.
            if float then each parameter will be sampled from a normal distribution with std of subj_noise.
            if dictionary then only parameters that are keys of subj_noise will be sampled, and the std of the sampling
            distribution will be the value associated with them.
        exclude_params : tuple <default ()>
            Do not add noise to these parameters.
        check_valid_func : function <default lambda x: True>
            Function that takes as input the parameters as kwds
            and returns True if param values are admissable.
        bounds : dict <default={}>
            Dict containing parameter names and
            (lower, upper) value for valid parameter
            range.
        column_name : str <default='data'>
            What to name the data column.

    :Returns:
        data : numpy structured array
            Will contain the columns 'subj_idx', 'condition' and 'data' which contains
            the random samples.
        subj_params : dict mapping condition to list of individual subject parameters
            Tries to be smart and will return direct values if there is only 1 subject
            and no dict if there is only 1 condition.

    """
    from itertools import product

    # Check if only dict of params was passed, i.e. no conditions
    if not isinstance(params[params.keys()[0]], dict):
        params = {'none': params}


    subj_params = {}
    dtype = Stochastic('temp', size=2, **(params.values()[0])).dtype
    if seed is not None:
        np.random.seed(seed)

    idx = list(product(range(subjs), params.keys(), range(size)))
    data = np.array(idx, dtype=[('subj_idx', np.int32), ('condition', 'S20'), (column_name, dtype)])

    for condition, param in params.iteritems():
        subj_params[condition] = []
        for subj_idx in range(subjs):
            if subjs > 1:
                # Sample subject parameters from a normal around the specified parameters
                subj_param = _add_noise(param, noise=subj_noise,
                                        check_valid_func=check_valid_func,
                                        bounds=bounds,
                                        exclude_params=exclude_params)
            else:
                subj_param = param
            subj_params[condition].append(subj_param)
            samples_from_dist = Stochastic('temp', size=size, **subj_param).value
            idx = (data['subj_idx'] == subj_idx) & (data['condition'] == condition)
            data[column_name][idx] = np.array(samples_from_dist, dtype=dtype)

    # Remove list around subj_params if there is only 1 subject
    if subjs == 1:
        for key, val in subj_params.iteritems():
            subj_params[key] = val[0]

    # Remove dict around subj_params if there is only 1 condition
    if len(subj_params) == 1:
        subj_params = subj_params[subj_params.keys()[0]]

    return data, subj_params




import numpy as np
import kabuki
import pymc as pm
import os.path
from copy import deepcopy
import matplotlib.pyplot as plt

def sort_dict(d):
    from operator import itemgetter
    return sorted(d.iteritems(), key=itemgetter(1))


def _model_type_to_class(model_type):
    """Takes either a (inherited) class of Hierarchical or a str with
    the name of the class and returns the class.

    """
    if isinstance(model_type, kabuki.Hierarchical):
        model_class = model_type
    elif isinstance(model_type, str):
        model_class = kabuki.utils.find_object(model_type)
    else:
        raise TypeError('Model type %s not supported.' % type(model_type))

    return model_class


def _parse_experiment(experiment):
    data = experiment['data']
    model_type = experiment['model_type']

    if 'kwargs' in experiment:
        kwargs = deepcopy(experiment['kwargs'])
    else:
        kwargs = {}


    if 'name' in experiment:
        name = experiment['name']
    else:
        name = model_type + str(sort_dict(kwargs))

    model_class = _model_type_to_class(model_type)

    return data, model_class, kwargs, name


def run_experiment(experiment, db='sqlite', samples=10000, burn=5000, thin=3, subj_map_init=True):
    """Run a single experiment: Builds the model, initializes,
    samples. See analyze_experiment() for how to generate output
    statistics of your finished model run.

    :Arguments:
        experiment : dict
            dict containing the following mappings:
            * 'data' -> np.ndarray
            * 'model_type' -> Hierarchical class or str name of class
            * 'name' -> str of name of experiment (optional)
            * 'kwargs' -> Keyword arguments to be supplied at model creation
        db : str (default='sqlite')
            Which database backend to use
        samples : int (default=10000)
            How many posterior samples to draw
        burn : int (default=5000)
            How many posterior samples to throw away as burn-in
        thin : int (default=3)
            How much thinning to apply
        subj_map_init : bool (default=True)
            Whether to initialize the model using subj_by_subj_map_init().

    :Returns:
        Str of summary statistics.

    """
    import os

    data, model_class, kwargs, name = _parse_experiment(experiment)

    if not os.path.exists(name):
        os.mkdir(name)

    m = model_class(data, **kwargs)

    if subj_map_init:
        m.subj_by_subj_map_init()

    m.mcmc(db=db, dbname=os.path.join(name, 'traces.db'))
    m.sample(samples, burn=burn, thin=thin)

    stats = kabuki.analyze.gen_stats(m.mc.stats())

    with open('%s/stats.txt'%name, 'w') as f:
        f.write("%f\n" % m.mc.dic)
        f.write(stats)

    return stats


def run_experiments(experiments, mpi=False, **kwargs):
    """Run a list of experiments. Optionally using MPI.

    :Arguments:
        experiments : list of dicts
            One entry for each experiment to run. Each experiment should have a dict like this:
            * 'data' -> np.ndarray
            * 'model_type' -> Hierarchical class or str name of class
            * 'name' -> str of name of experiment (optional)
            * 'kwargs' -> Keyword arguments to be supplied at model creation
        mpi : bool (default=False)
            Whether to run experiments in parallel using MPI. If set to True you must call
            your script (which calls this function) using mpirun.
            This requires mpi4py_map and mpi4py to be installed.
            E.g. pip install mpi4py mpi4py_map
        kwargs : dict
            Additional keyword arguments to be passed to run_experiment (see run_experiment),
            such as samples, burn etc.
    """
    if mpi:
        import mpi4py_map
        results = mpi4py_map.map(run_experiment, experiments, **kwargs)
    else:
        results = [run_experiment(experiment, **kwargs) for experiment in experiments]

    return results


def load_experiment(experiment, dbname='traces.db', db='sqlite'):
    """Load specific experiment from a database file.

    :Arguments:
        experiment : dict (see run_experiment for the contents)
        dbname : str (default='traces.db')
            Name of the database file.
        db : str (default='sqlite')
            Which database backend to use, can be
            sqlite, pickle, hdf5, txt.

    :Return:
        Same experiment with a new 'model' key linking to the loaded model.
    """

    experiment['model'] = load_model(experiment, db=db, dbname=dbname)
    return experiment

def load_model(experiment, db='sqlite', dbname='traces.db'):
    data, model_class, kwargs, name = _parse_experiment(experiment)
    m = model_class(data, **kwargs)

    m.load_db(os.path.join(name, dbname), db='sqlite')

    return m

def load_ppc(experiment):
    import pandas as pd
    data, model_class, kwargs, name = _parse_experiment(experiment)
    post_pred = pd.read_csv(os.path.join(name, 'post_pred.csv'))

    return post_pred

def load_ppcs(experiments):
    import pandas as pd
    post_preds = []
    for experiment in experiments:
        try:
            ppc = load_ppc(experiment)
        except:
            pass
        post_preds.append(ppc)

    model_names = [_parse_experiment(experiment)[-1] for experiment in experiments]
    print model_names
    return (model_names, post_preds)
    #return pd.concat(post_preds, keys=model_names, names=['model'])

def analyze_experiment(experiment, plot_groups=True, plot_traces=True, plot_post_pred=True, ppc=True, stats=None):
    """Analyze a single experiment. Writes output statitics and various plots into a subdirectory.

    :Arguments:
        experiment : dict
            Should be a dict like this:
            * 'data' -> np.ndarray
            * 'model_type' -> Hierarchical class or str name of class
            * 'name' -> str of name of experiment (optional)
            * 'kwargs' -> Keyword arguments to be supplied at model creation
        plot_groups : bool (default=True)
            Whether to plot all traces in a group plot.
        plot_traces : bool (default=True)
            Whether to plot all traces in a trace plot.
        plot_post_pred : bool (default=True)
            Whether to plot the posterior predictive.
        ppc : bool (default=True)
            Whether to run a posterior predictive check.
        stats : dict (default=None)
            The stats to be used for the posterior predictive check (see kabuki.analyze.post_pred_check)
    """
    data, model_class, kwargs, name = _parse_experiment(experiment)

    if 'model' in experiment:
        # Model already loaded, use it
        model = experiment['model']
    else: # Load it
        model = load_model(experiment)

    print "Analyzing model: %s" % name
    if plot_groups:
        kabuki.analyze.group_plot(model, save_to=name)

    if plot_traces:
        model.plot_posteriors(path=name)

    if plot_post_pred:
        print "Plotting posterior predictive"
        kabuki.analyze.plot_posterior_predictive(model, np.linspace(-1.2, 1.2, 80), savefig=True, path=name, columns=7, figsize=(18,18))

    if ppc:
        ppc = kabuki.analyze.post_pred_check(model, stats=stats)
        print ppc
        ppc.to_csv(os.path.join(name, 'post_pred.csv'))


def analyze_experiments(experiments, mpi=False, plot_dic=True, **kwargs):
    """Analyze multiple experiments. Outputs will be saved to
    subdirectories. Can optionally analyze in parallel.

    :Arguments:
        experiments : list of dicts
            One entry for each experiment to analyze. Each experiment should have a dict like this:
            * 'data' -> np.ndarray
            * 'model_type' -> Hierarchical class or str name of class
            * 'name' -> str of name of experiment (optional)
            * 'kwargs' -> Keyword arguments to be supplied at model creation
        mpi : bool (default=False)
            Whether to run experiments in parallel using MPI. If set to True you must call
            your script (which calls this function) using mpirun.
            This requires mpi4py_map and mpi4py to be installed.
            E.g. pip install mpi4py mpi4py_map
        plot_dic : bool (default=True)
            Whether to create a bar plot of DIC values for each model.
        kwargs : dict
            Keyword arguments to be passed to analyze_experiment. See analyze_experiment.

    """

    # Load models if necessary
    for experiment in experiments:
        if 'model' not in experiment:
            experiment['model'] = load_model(experiment)

    if mpi:
        import mpi4py_map
        results = mpi4py_map.map(analyze_experiment, experiments, **kwargs)
    else:
        results = [analyze_experiment(experiment, **kwargs) for experiment in experiments]

    if plot_dic:
        dics = [experiment['model'].mc.dic for experiment in experiments]
        names = [_parse_experiment(experiment)[-1] for experiment in experiments]

        fig = plt.figure()
        x = range(len(names))
        ax = plt.bar(x, dics, align='center')
        plt.xticks(x, names)
        plt.ylabel('DIC')
        fig.autofmt_xdate()
        fig.savefig('dic.png')
        fig.savefig('dic.pdf')

    return results

if __name__=='__main__':
    from copy import copy

    # Example, requires HDDM
    import hddm

    # Generate data
    params_single = hddm.generate.gen_rand_params()
    params = {'cond1': copy(params_single), 'cond2': copy(params_single)}
    params['cond2']['v'] = 0
    data, subj_params = kabuki.generate.gen_rand_data(hddm.likelihoods.Wfpt,
                                         params=params, samples=100, subjs=10, column_name='rt')[0]

    # Create different models to test our various hypotheses.
    experiments = [
                   {'name': 'baseline', 'data': data, 'model_type': 'hddm.HDDM', 'kwargs': {'depends_on': {}}},
                   {'name': 'condition_influences_drift', 'data': data, 'model_type': 'hddm.HDDM', 'kwargs': {'depends_on': {'v': 'condition'}}},
                   {'name': 'condition_influences_threshold', 'data': data, 'model_type': 'hddm.HDDM', 'kwargs': {'depends_on': {'a': 'condition'}}}]

    print "Running experiments..."
    run_experiments(experiments)

    print "Analyzing experiments..."
    analyze_experiments(experiments, ppc=False)

    print "Done! Check the newly created subdirectories."





#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
import numpy.lib.recfunctions as rec
import pymc as pm
from copy import copy
try:
    from IPython.Debugger import Tracer; debug_here = Tracer()
except:
    def debug_here(): pass


import kabuki
from kabuki.hierarchical import Parameter

from pymc.NumpyDeterministics import deterministic_from_funcs

def where_wrap(condition, x, y):
    """Wrapper for where function"""
    return np.where(condition, x, y) # because numpy functions often don't allow keyword args

stoch_where = deterministic_from_funcs('where',
                                       where_wrap,
                                       jacobians = {'x': lambda condition, x, y: np.where(condition, 1, 0),
                                                    'y': lambda condition, x, y: np.where(condition, 0, 1)},
                                       jacobian_formats = {'x': 'broadcast_operation', 'y': 'broadcast_operation' })

def q_learn(q_mat, lrate, stim, action, reward):
    #if type(q_mat) is not np.ndarray:
    #    q_mat = q_mat.parents['x']
    #select_action = np.zeros_like(q_mat)
    #select_action[stim,action] = 1

    #return stoch_where(select_action, q_mat, q_mat + lrate*(reward-q_mat))

    q_mat_out=np.copy(q_mat)
    # Calc Q-value
    q_val = q_mat[stim,action]
    q_mat_out[stim,action] = q_val + lrate*(reward-q_val)
    return q_mat_out

def softmax(q_mat, inv_temp, stim, action):
    #sm = pm.exp(pm.exp(inv_temp)*q_mat[stim,action]) / (pm.sum(pm.exp(pm.exp(inv_temp)*q_mat[stim,:])))
    sm = np.exp(inv_temp*q_mat[stim,action]) / (np.sum(np.exp(inv_temp*q_mat[stim,:])))
    return sm

def create_choice_probs(data=None, inv_temp=None, lrate=None):
    # Run model
    size = len(data)
    choice_probs = np.empty(size)
    q_vals = []

    for t in range(size):
        if t==0:
            q_val = .5*np.ones((2,2))
        else:
            q_val = q_vals[-1]

        choice_probs[t] = softmax(q_val, inv_temp, data['stim'][t], data['action'][t])
        q_vals.append(q_learn(q_val, lrate, data['stim'][t], data['action'][t], data['reward'][t]))

    return choice_probs

class QLearn(kabuki.Hierarchical):
    def __init__(self, data, **kwargs):
        super(QLearn, self).__init__(data, **kwargs)

        self.num_stims = len(np.unique(data['stim']))
        self.num_actions = len(np.unique(data['action']))

        self.params = [Parameter('lrate', lower=0, upper=1, init=.1),
                       Parameter('inv_temp', lower=1, upper=15, init=5),
                       Parameter('choice_probs', is_bottom_node=True),
                       Parameter('like', is_bottom_node=True)]

    def get_var_node(self, param):
        if param.name == 'lrate':
            return pm.Uniform(param.full_name, lower=0.1, upper=2.,
                              value=1., plot=self.plot_var)
        elif param.name == 'inv_temp':
            return pm.Uniform(param.full_name, lower=.5, upper=5.,
                              value=3., plot=self.plot_var)

    def get_bottom_node(self, param, params):
        #q_vals = np.empty(len(param.data), dtype=object)
        #softmax_probs = np.empty(len(param.data), dtype=object)
        #choice_probs = np.empty(len(param.data), dtype=object)

        if param.name == 'choice_probs':
            return pm.Deterministic(create_choice_probs,
                                    param.full_name,
                                    param.full_name,
                                    {'data':param.data,
                                     'inv_temp':params['inv_temp'],
                                     'lrate':params['lrate']})

        elif param.name == 'like':
            # Create likelihood
            return pm.Bernoulli(param.full_name,
                                p=params['choice_probs'],
                                value=np.ones_like(np.ones_like(params['choice_probs'].value)),
                                observed=True)

        else:
            print "Not found."

#########
# Generate

import numpy as np
import matplotlib.pyplot as plt

def qlearn_generate(trials=50, lrate=.1, rew_prob=.7, inv_temp=5., plot=False, subjs=2):
    Q = .5*np.ones((2,2))
    out = np.empty(trials*subjs, dtype=[('subj_idx',np.int), ('stim', np.int), ('action', np.int), ('reward', np.float)])
    Q1 = []
    Q2 = []
    for subj in range(subjs):
        idx = slice(subj*trials, (subj+1)*trials)
        out[idx]['subj_idx'] = subj
        for t in range(trials):
            # Present stimulus 0 or 1 (alternating)
            state = t%2
            # Chose action based on softmax over Q-values
            action = softmax_gen(Q[state], inv_temp)

            # Reward is probabilistic (70% for a1 to s1 and vice versa)
            rnd = np.random.rand()
            if action == state:
                reward = 1 if rnd < rew_prob else 0
            else:
                reward = 0 if rnd < rew_prob else 1

            # Update Q-value based on reward received
            Q[state,action] = Q[state,action] + lrate * (reward - Q[state,action])

            out[idx]['stim'][t] = state
            out[idx]['action'][t] = action
            out[idx]['reward'][t] = reward

            Q1.append(Q[0,0])
            Q2.append(Q[0,1])

    if plot:
        plt.figure()
        plt.plot(Q1)
        plt.plot(Q2)

    return out


def softmax_gen(Q1, inv_temp):
    p1 = np.exp(Q1[0]*inv_temp)
    p1 = p1 / (p1 + np.exp(Q1[1]*inv_temp))

    rand = np.random.rand()

    return 0 if rand < p1 else 1



